package jumpstart.web.pages.examples.ajax;

import java.text.DateFormat;
import java.text.Format;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;
import jumpstart.web.pages.Index;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.ValueEncoder;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Persist;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;

public class AjaxFormLoop1 {
	private final int MAX_RESULTS = 30;
	private final String NEW_PERSON = "New";

	// Screen fields

	@Property
	private List<Person> persons;

	@Property
	@SuppressWarnings("unused")
	private Person person;

	@Property
	@SuppressWarnings("unused")
	private PersonEncoder personEncoder;

	// Work fields

	// We are using "session" persistence but it is risky. Better techniques include wrapping this field in a
	// Conversation (see the Wizard examples) or persisting it in the database.
	@Persist
	private List<Person> personsToDelete;

	private List<Person> personsToCreate;
	private List<Person> personsToChange;
	private boolean oneOrMorePersonsGone;

	// Other pages

	@InjectPage
	private AjaxFormLoop2 page2;

	// Generally useful bits and pieces

	@Component(id = "personsedit")
	private Form form;

	@EJB
	private IPersonServiceLocal personService;

	@Inject
	private Locale currentLocale;

	@Inject
	private Request request;

	@Inject
	private ComponentResources componentResources;

	// The code

	void onActivate() {

		// The following initialization would usually be done in onPrepareForSubmit() or onPrepare(), but we do it here
		// just in case we got here by the Back button followed by an Ajax request.

		personEncoder = new PersonEncoder();

		if (personsToDelete == null) {
			personsToDelete = new ArrayList<Person>();
		}
	}

	// Form bubbles up the PREPARE_FOR_SUBMIT event during form submission.

	void onPrepareForSubmit() {
		personsToCreate = new ArrayList<Person>();
		personsToChange = new ArrayList<Person>();
		oneOrMorePersonsGone = false;
	}

	// Form bubbles up the PREPARE event during form render and form submission.

	void onPrepare() {
		if (persons == null) {
			// Get all persons - ask business service to find them (from the database)
			persons = personService.findPersons(MAX_RESULTS);

			personEncoder = new PersonEncoder();
		}
	}

	// This encoder is used by our AjaxFormLoop:
	// - during render, to convert each person to an id (AjaxFormLoop then stores the ids in the form, hidden).
	// - during form submission, to convert each id back to a person which it puts in our person field. We
	// also add it to either personsToChange or personsToCreate. AjaxFormLoop will overwrite several fields of the
	// person returned.

	private class PersonEncoder implements ValueEncoder<Person> {

		public String toClient(Person value) {
			Long id = value.getId();
			return id == null ? NEW_PERSON : id.toString();
		}
		public Person toValue(String idAsString) {
			Person person = null;
			Long id = null;

			if (!idAsString.equals(NEW_PERSON)) {
				id = new Long(idAsString);
			}

			// If new person, return an empty person.

			if (id == null) {
				person = new Person();

				// If we are in form submission, then add person to personsToCreate

				if (!request.isXHR()) {
					personsToCreate.add(person);
				}
			}

			// Else, return existing person

			else {
				person = findPerson(id);

				// If we are in form submission, then add person to personsToChange unless they are gone from the
				// database

				if (!request.isXHR()) {
					if (person == null) {
						oneOrMorePersonsGone = true;
					}
					else {
						personsToChange.add(person);
					}
				}
			}

			// AjaxFormLoop will overwrite several fields of the person returned.
			// AjaxFormLoop can't handle null person, so if null we return a new empty person.
			return person == null ? new Person() : person;
		}

		private Person findPerson(Long id) {

			if (request.isXHR()) {
				return personService.findPerson(id);
			}
			else {
				// We could find the person in the database, but it's cheaper to search the list we got in onPrepare().

				for (Person person : persons) {
					if (person.getId().equals(id)) {
						return person;
					}
				}
			}
			return null;
		}
	};

	Person onAddRow() {
		// Return a empty person
		return new Person();
	}

	void onRemoveRow(Person person) {

		// If not a new person, add them to personsToDelete.

		if (person.getId() != null) {
			personsToDelete.add(person);
		}
	}

	void onValidateFromPersonsEdit() {

		if (form.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		// Let's say a requirement is to not proceed if a person we displayed is no longer in the database - check it.

		if (oneOrMorePersonsGone) {
			form.recordError("The list of persons is out of date. Please refresh and try again.");
			return;
		}

		System.out.println(">>> personsToCreate = " + personsToCreate);
		System.out.println(">>> personsToChange = " + personsToChange);
		System.out.println(">>> personsToDelete = " + personsToDelete);

		try {
			// In a real application you would persist them to the database instead of printing them
			// personService.bulkEditPersons(personsToCreate, personsToChange, personsToDelete);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			form.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	Object onSuccess() {
		List<Person> persons = new ArrayList<Person>();

		persons.addAll(personsToChange);
		persons.addAll(personsToCreate);

		page2.set(persons, personsToDelete);
		componentResources.discardPersistentFieldChanges();
		return page2;
	}

	void onRefresh() {
		componentResources.discardPersistentFieldChanges();
		onPrepare();
	}

	Object onGoHome() {
		componentResources.discardPersistentFieldChanges();
		return Index.class;
	}

	public Format getDateFormat() {
		return DateFormat.getDateInstance(DateFormat.SHORT, currentLocale);
	}

}
