package jumpstart.web.pages.examples.ajax;

import java.text.DateFormat;
import java.text.Format;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.ejb.EJB;

import jumpstart.business.domain.examples.Person;
import jumpstart.business.domain.examples.Regions;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.util.ExceptionUtil;
import jumpstart.web.pages.Index;

import org.apache.tapestry5.ComponentResources;
import org.apache.tapestry5.ValueEncoder;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.InjectPage;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.corelib.components.Form;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.Request;

public class AjaxFormLoopTailored1 {
	private final int MAX_RESULTS = 30;
	private final String NEW_PERSON = "New";

	// Screen fields

	@Property
	private List<Person> persons;

	@Property
	private Person person;

	@Property
	@SuppressWarnings("unused")
	private PersonEncoder personEncoder;

	// Work fields

	private List<Person> personsToCreate;

	// Other pages

	@InjectPage
	private AjaxFormLoopTailored2 page2;

	// Generally useful bits and pieces

	@Component(id = "personsedit")
	private Form form;

	@EJB
	private IPersonServiceLocal personService;

	@Inject
	private Messages messages;

	@Inject
	private Locale currentLocale;

	@Inject
	private Request request;

	@Inject
	private ComponentResources componentResources;

	// The code

	void onActivate() {
		personEncoder = new PersonEncoder();
	}

	// Form bubbles up the PREPARE_FOR_SUBMIT event during form submission.

	void onPrepareForSubmit() {
		personsToCreate = new ArrayList<Person>();
	}

	// Form bubbles up the PREPARE event during form render and form submission.

	void onPrepare() {
		if (persons == null) {
			// Get all persons - ask business service to find them (from the database)
			persons = personService.findPersons(MAX_RESULTS);

			personEncoder = new PersonEncoder();
		}
	}

	// This encoder is used by our AjaxFormLoop:
	// - during render, to convert each person to an id (AjaxFormLoop then stores the ids in the form, hidden).
	// - during form submission, to convert each id back to a person which it puts in our person field. We
	// also add it to either personsToChange or personsToCreate. AjaxFormLoop will overwrite several fields of the
	// person returned.

	private class PersonEncoder implements ValueEncoder<Person> {

		public String toClient(Person value) {
			Long id = value.getId();
			return id == null ? NEW_PERSON : id.toString();
		}
		public Person toValue(String idAsString) {
			Person person = null;
			Long id = null;

			if (!idAsString.equals(NEW_PERSON)) {
				id = new Long(idAsString);
			}

			// If new person, return an empty person.

			if (id == null) {
				person = new Person();

				// If we are in form submission, then add person to personsToCreate

				if (!request.isXHR()) {
					personsToCreate.add(person);
				}
			}

			// Else, return existing person

			else {
				person = findPerson(id);
			}

			// AjaxFormLoop will overwrite several fields of the person returned.
			// AjaxFormLoop can't handle null person, so if null we return a new empty person.
			return person == null ? new Person() : person;
		}

		private Person findPerson(Long id) {

			if (request.isXHR()) {
				return personService.findPerson(id);
			}
			else {
				// We could find the person in the database, but it's cheaper to search the list we got in onPrepare().

				for (Person person : persons) {
					if (person.getId().equals(id)) {
						return person;
					}
				}
			}
			return null;
		}
	};

	Person onAddRow() {
		// Return a empty person
		return new Person();
	}

	void onRemoveRow(Person person) {
		// Nothing to do
	}

	void onValidateFromPersonsEdit() {

		if (form.getHasErrors()) {
			// We get here only if a server-side validator detected an error.
			return;
		}

		System.out.println(">>> personsToCreate = " + personsToCreate);

		try {
			// In a real application you would persist them to the database instead of printing them
			// personService.createPersons(personsToCreate);
		}
		catch (Exception e) {
			// Display the cause. In a real system we would try harder to get a user-friendly message.
			form.recordError(ExceptionUtil.getRootCauseMessage(e));
		}
	}

	Object onSuccess() {
		page2.set(personsToCreate);
		componentResources.discardPersistentFieldChanges();
		return page2;
	}

	void onRefresh() {
		componentResources.discardPersistentFieldChanges();
		onPrepare();
	}

	Object onGoHome() {
		componentResources.discardPersistentFieldChanges();
		return Index.class;
	}
	public boolean isNewPerson() {
		return person.getId() == null;
	}

	public String getPersonRegion() {
		// Follow the same naming convention that the Select component uses
		return messages.get(Regions.class.getSimpleName() + "." + person.getRegion().name());
	}

	public Format getDateFormat() {
		return DateFormat.getDateInstance(DateFormat.SHORT, currentLocale);
	}

}
